@@ -4,6 +4,7 @@ import (
 	"bufio"
 	"fmt"
 	"io"
+	"io/fs"
 	"io/ioutil"
 	"log"
 	"net/url"
@@ -13,6 +14,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"runtime/pprof"
 	"sort"
 	"strings"
 	"time"
@@ -29,9 +31,8 @@ import (
 	"github.com/tdewolff/minify/v2/xml"
 )
 
-var Version = "master"
-var Commit = ""
-var Date = ""
+// Version is the current minify version.
+var Version = "built from source"
 
 var filetypeMime = map[string]string{
 	"css":  "text/css",
@@ -44,46 +45,60 @@ var filetypeMime = map[string]string{
 }
 
 var (
-	help      bool
-	hidden    bool
-	list      bool
-	m         *min.M
-	pattern   *regexp.Regexp
-	recursive bool
-	verbose   bool
-	version   bool
-	watch     bool
-	sync      bool
+	help             bool
+	hidden           bool
+	list             bool
+	m                *min.M
+	pattern          *regexp.Regexp
+	recursive        bool
+	verbose          bool
+	version          bool
+	watch            bool
+	sync             bool
+	bundle           bool
+	preserveSymlinks bool
+	mimetype         string
 )
 
+// Task is a minify task.
 type Task struct {
 	srcs []string
 	dst  string
+	sync bool
 }
 
-func NewTask(root, input, output string) Task {
-	t := Task{[]string{input}, output}
+// NewTask returns a new Task.
+func NewTask(root, input, output string, sync bool) (Task, error) {
+	t := Task{[]string{input}, output, sync}
 	if 0 < len(output) && output[len(output)-1] == '/' {
 		rel, err := filepath.Rel(root, input)
 		if err != nil {
-			Error.Fatalln(err)
+			return Task{}, err
 		}
-		t.dst = path.Clean(filepath.ToSlash(path.Join(output, rel)))
+		t.dst = path.Join(output, path.Base(root), rel)
 	}
-	return t
+	return t, nil
 }
 
+// Loggers.
 var (
-	Error *log.Logger
-	Info  *log.Logger
+	Error   *log.Logger
+	Warning *log.Logger
+	Info    *log.Logger
 )
 
 func main() {
+	// os.Exit doesn't execute pending defer calls, this is fixed by encapsulating run()
+	os.Exit(run())
+}
+
+func run() int {
 	output := ""
-	mimetype := ""
 	filetype := ""
 	match := ""
 	siteurl := ""
+	cpuprofile := ""
+	memprofile := ""
 
 	cssMinifier := &css.Minifier{}
 	htmlMinifier := &html.Minifier{}
@@ -96,12 +111,12 @@ func main() {
 	flag.Usage = func() {
 		fmt.Fprintf(os.Stderr, "Usage: %s [options] [input]\n\nOptions:\n", os.Args[0])
 		flag.PrintDefaults()
-		fmt.Fprintf(os.Stderr, "\nInput:\n  Files or directories, leave blank to use stdin\n")
+		fmt.Fprintf(os.Stderr, "\nInput:\n  Files or directories, leave blank to use stdin. Specify --mime or --type to use stdin and stdout.\n")
 	}
 
 	flag.BoolVarP(&help, "help", "h", false, "Show usage")
 	flag.StringVarP(&output, "output", "o", "", "Output file or directory (must have trailing slash), leave blank to use stdout")
-	flag.StringVar(&mimetype, "mime", "", "Mimetype (eg. text/css), optional for input filenames, has precedence over -type")
+	flag.StringVar(&mimetype, "mime", "", "Mimetype (eg. text/css), optional for input filenames, has precedence over --type")
 	flag.StringVar(&filetype, "type", "", "Filetype (eg. css), optional for input filenames")
 	flag.StringVar(&match, "match", "", "Filename pattern matching using regular expressions")
 	flag.BoolVarP(&recursive, "recursive", "r", false, "Recursively minify directories")
@@ -110,47 +125,54 @@ func main() {
 	flag.BoolVarP(&verbose, "verbose", "v", false, "Verbose")
 	flag.BoolVarP(&watch, "watch", "w", false, "Watch files and minify upon changes")
 	flag.BoolVarP(&sync, "sync", "s", false, "Copy all files to destination directory and minify when filetype matches")
+	flag.BoolVarP(&preserveSymlinks, "preserve-links", "p", false, "Copy symbolic links without dereferencing and without minifying the referenced file (only with --sync)")
+	flag.BoolVarP(&bundle, "bundle", "b", false, "Bundle files by concatenation into a single file")
 	flag.BoolVarP(&version, "version", "", false, "Version")
 
 	flag.StringVar(&siteurl, "url", "", "URL of file to enable URL minification")
-	flag.IntVar(&cssMinifier.Decimals, "css-decimals", 0, "Number of significant digits to preserve in numbers, 0 is all (DEPRECATED")
+	flag.StringVar(&cpuprofile, "cpuprofile", "", "Export CPU profile")
+	flag.StringVar(&memprofile, "memprofile", "", "Export memory profile")
 	flag.IntVar(&cssMinifier.Precision, "css-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
+	flag.BoolVar(&htmlMinifier.KeepComments, "html-keep-comments", false, "Preserve all comments")
 	flag.BoolVar(&htmlMinifier.KeepConditionalComments, "html-keep-conditional-comments", false, "Preserve all IE conditional comments")
 	flag.BoolVar(&htmlMinifier.KeepDefaultAttrVals, "html-keep-default-attrvals", false, "Preserve default attribute values")
 	flag.BoolVar(&htmlMinifier.KeepDocumentTags, "html-keep-document-tags", false, "Preserve html, head and body tags")
 	flag.BoolVar(&htmlMinifier.KeepEndTags, "html-keep-end-tags", false, "Preserve all end tags")
 	flag.BoolVar(&htmlMinifier.KeepWhitespace, "html-keep-whitespace", false, "Preserve whitespace characters but still collapse multiple into one")
 	flag.BoolVar(&htmlMinifier.KeepQuotes, "html-keep-quotes", false, "Preserve quotes around attribute values")
-	flag.IntVar(&svgMinifier.Decimals, "svg-decimals", 0, "Number of significant digits to preserve in numbers, 0 is all (DEPRECATED")
+	flag.IntVar(&jsMinifier.Precision, "js-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
+	flag.BoolVar(&jsMinifier.KeepVarNames, "js-keep-var-names", false, "Preserve original variable names")
+	flag.IntVar(&jsonMinifier.Precision, "json-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
 	flag.IntVar(&svgMinifier.Precision, "svg-precision", 0, "Number of significant digits to preserve in numbers, 0 is all")
 	flag.BoolVar(&xmlMinifier.KeepWhitespace, "xml-keep-whitespace", false, "Preserve whitespace characters but still collapse multiple into one")
-	if err := flag.Parse(os.Args[1:]); err != nil {
-		fmt.Printf("Error: %v\n\n", err)
-		flag.Usage()
-		os.Exit(2)
+	if len(os.Args) == 1 {
+		fmt.Printf("minify: must specify --mime or --type in order to use stdin and stdout\n")
+		fmt.Printf("Try 'minify --help' for more information\n")
+		return 1
+	} else if err := flag.Parse(os.Args[1:]); err != nil {
+		fmt.Printf("minify: %v\n", err)
+		fmt.Printf("Try 'minify --help' for more information\n")
+		return 1
 	}
-	rawInputs := flag.Args()
-	useStdin := len(rawInputs) == 0
+	inputs := flag.Args()
+	useStdin := len(inputs) == 0
 
 	Error = log.New(os.Stderr, "ERROR: ", 0)
+	Warning = log.New(os.Stderr, "WARNING: ", 0)
 	if verbose {
-		Info = log.New(os.Stderr, "INFO: ", 0)
+		Info = log.New(os.Stderr, "", 0)
 	} else {
-		Info = log.New(ioutil.Discard, "INFO: ", 0)
+		Info = log.New(ioutil.Discard, "", 0)
 	}
 
 	if help {
 		flag.Usage()
-		os.Exit(0)
+		return 0
 	}
 
 	if version {
-		if Version == "devel" {
-			fmt.Printf("minify version devel+%.7s %s\n", Commit, Date)
-		} else {
-			fmt.Printf("minify version %s\n", Version)
-		}
-		os.Exit(0)
+		fmt.Printf("minify %s\n", Version)
+		return 0
 	}
 
 	if list {
@@ -162,14 +184,57 @@ func main() {
 		for _, k := range keys {
 			fmt.Println(k + "\t" + filetypeMime[k])
 		}
-		os.Exit(0)
+		return 0
+	}
+
+	if cpuprofile != "" {
+		f, err := os.Create(cpuprofile)
+		if err != nil {
+			Error.Println(err)
+			return 1
+		}
+		if err = pprof.StartCPUProfile(f); err != nil {
+			Error.Println(err)
+			return 1
+		}
+		defer func() {
+			pprof.StopCPUProfile()
+			if err = f.Close(); err != nil {
+				Error.Println(err)
+			}
+		}()
+	}
+
+	if memprofile != "" {
+		defer func() {
+			f, err := os.Create(memprofile)
+			if err != nil {
+				Error.Println(err)
+			}
+			if err = pprof.WriteHeapProfile(f); err != nil {
+				Error.Println(err)
+			}
+			if err = f.Close(); err != nil {
+				Error.Println(err)
+			}
+		}()
 	}
 
 	var err error
 	if match != "" {
 		pattern, err = regexp.Compile(match)
 		if err != nil {
-			Error.Fatalln(err)
+			Error.Println(err)
+			return 1
+		}
+	}
+
+	// detect mimetype, mimetype=="" means we'll infer mimetype from file extensions
+	if mimetype == "" && filetype != "" {
+		var ok bool
+		if mimetype, ok = filetypeMime[filetype]; !ok {
+			Error.Println("cannot find mimetype for filetype", filetype)
+			return 1
 		}
 	}
 
@@ -183,18 +248,14 @@ func main() {
 		if recursive {
 			Error.Println("--recursive doesn't work on stdin and stdout, specify input and output")
 		}
-		os.Exit(1)
+		return 1
 	}
-	if mimetype == "" && filetype == "" && useStdin {
-		Error.Fatalln("must specify --mime or --type for stdin")
-	}
-
-	// detect mimetype, mimetype=="" means we'll infer mimetype from file extensions
-	if mimetype == "" && filetype != "" {
-		var ok bool
-		if mimetype, ok = filetypeMime[filetype]; !ok {
-			Error.Fatalln("cannot find mimetype for filetype", filetype)
-		}
+	if mimetype == "" && useStdin {
+		Error.Println("must specify --mime or --type for stdin")
+		return 1
+	} else if mimetype != "" && sync {
+		Error.Println("must specify either --sync or --mime/--type")
+		return 1
 	}
 	if verbose {
 		if mimetype == "" {
@@ -206,26 +267,24 @@ func main() {
 
 	////////////////
 
+	for i, input := range inputs {
+		inputs[i] = sanitizePath(input)
+	}
+
 	// set output, empty means stdout, ending in slash means a directory, otherwise a file
 	dirDst := false
 	if output != "" {
+		dirSrc := 1 < len(inputs) || len(inputs) == 1 && inputs[0][len(inputs[0])-1] == '/'
+		if !bundle && dirSrc && output[len(output)-1] != '/' {
+			output += "/"
+		}
 		output = sanitizePath(output)
-		if output[len(output)-1] == '/' {
-			dirDst = true
-			if err := os.MkdirAll(output, 0777); err != nil {
-				Error.Fatalln(err)
-			}
+		dirDst = output[len(output)-1] == '/'
+		if dirDst && bundle {
+			Error.Println("--bundle requires destination to be stdout or a file")
+			return 1
 		}
 	}
-	if !dirDst && (sync || watch) {
-		if sync {
-			Error.Println("--sync requires destination to be a directory")
-		}
-		if watch {
-			Error.Println("--watch requires destination to be a directory")
-		}
-		os.Exit(1)
-	}
 	if verbose {
 		if output == "" {
 			Info.Println("minify to stdout")
@@ -240,18 +299,34 @@ func main() {
 			Info.Println("minify from stdin")
 		}
 	}
+	if dirDst {
+		if err := os.MkdirAll(output, 0777); err != nil {
+			Error.Println(err)
+			return 1
+		}
+	}
 
 	var tasks []Task
 	var roots []string
 	if useStdin {
-		tasks = append(tasks, NewTask("", "", output))
+		task, err := NewTask("", "", output, false)
+		if err != nil {
+			Error.Println(err)
+			return 1
+		}
+		tasks = append(tasks, task)
 		roots = append(roots, "")
 	} else {
-		tasks, roots = createTasks(rawInputs, output)
+		tasks, roots, err = createTasks(inputs, output)
+		if err != nil {
+			Error.Println(err)
+			return 1
+		}
 	}
 
 	// concatenate
-	if 1 < len(tasks) && !dirDst {
+	if 1 < len(tasks) && bundle {
+		// Task.sync == false because dirDst == false
 		for _, task := range tasks[1:] {
 			tasks[0].srcs = append(tasks[0].srcs, task.srcs[0])
 		}
@@ -269,7 +344,8 @@ func main() {
 	m.AddRegexp(regexp.MustCompile("[/+]xml$"), xmlMinifier)
 
 	if m.URL, err = url.Parse(siteurl); err != nil {
-		Error.Fatalln(err)
+		Error.Println(err)
+		return 1
 	}
 
 	numWorkers := 1
@@ -285,7 +361,7 @@ func main() {
 	chanTasks := make(chan Task, 100)
 	chanFails := make(chan int, numWorkers)
 	for n := 0; n < numWorkers; n++ {
-		go minifyWorker(mimetype, chanTasks, chanFails)
+		go minifyWorker(chanTasks, chanFails)
 	}
 
 	if !watch {
@@ -295,7 +371,8 @@ func main() {
 	} else {
 		watcher, err := NewWatcher(recursive)
 		if err != nil {
-			Error.Fatalln(err)
+			Error.Println(err)
+			return 1
 		}
 		defer watcher.Close()
 
@@ -305,9 +382,13 @@ func main() {
 		}
 
 		autoDir := false
-		for _, root := range roots {
-			watcher.AddPath(root)
-			if root == output {
+		files := roots
+		if !recursive {
+			files = inputs
+		}
+		for _, filename := range files {
+			watcher.AddPath(filename)
+			if filename == output {
 				autoDir = true
 			}
 		}
@@ -327,8 +408,8 @@ func main() {
 				file = sanitizePath(file)
 
 				// find longest common path among roots
-				root := roots[0]
-				for _, path := range roots[1:] {
+				root := ""
+				for _, path := range roots {
 					pathRel, err1 := filepath.Rel(path, file)
 					rootRel, err2 := filepath.Rel(root, file)
 					if err2 != nil || err1 == nil && len(pathRel) < len(rootRel) {
@@ -336,9 +417,9 @@ func main() {
 					}
 				}
 
-				// skip files in output directory (which is also an input directory) for the first change
-				// skips files that are not minified and stay put, as they are not explicitly copied, but that's ok
 				if autoDir && root == output {
+					// skip files in output directory (which is also an input directory) for the first change
+					// skips files that are not minified and stay put as they are not explicitly copied, but that's ok
 					if _, ok := skip[file]; !ok {
 						skip[file] = true
 						break
@@ -348,7 +429,12 @@ func main() {
 				if !verbose {
 					Info.Println(file, "changed")
 				}
-				chanTasks <- NewTask(root, file, output)
+				task, err := NewTask(root, file, output, !fileMatches(file))
+				if err != nil {
+					Error.Println(err)
+					return 1
+				}
+				chanTasks <- task
 			}
 		}
 	}
@@ -360,18 +446,18 @@ func main() {
 	}
 
 	if verbose && !watch {
-		Info.Println(time.Since(start), "total")
+		Info.Println("finished in", time.Since(start))
 	}
-	if fails > 0 {
-		os.Exit(1)
+	if 0 < fails {
+		return 1
 	}
-	os.Exit(0)
+	return 0
 }
 
-func minifyWorker(mimetype string, chanTasks <-chan Task, chanFails chan<- int) {
+func minifyWorker(chanTasks <-chan Task, chanFails chan<- int) {
 	fails := 0
 	for task := range chanTasks {
-		if ok := minify(mimetype, task); !ok {
+		if ok := minify(task); !ok {
 			fails++
 		}
 	}
@@ -379,87 +465,151 @@ func minifyWorker(mimetype string, chanTasks <-chan Task, chanFails chan<- int)
 }
 
 func sanitizePath(p string) string {
-	p = filepath.ToSlash(p)
-	isDir := p[len(p)-1] == '/'
+	p = filepath.ToSlash(p) // replace \ with / for Windows
+	isCur := path.Base(p) == "."
+	isDir := p[len(p)-1] == '/' || isCur
 	p = path.Clean(p)
+	if p == "." {
+		return "./"
+	}
 	if isDir {
 		p += "/"
-	} else if info, err := os.Stat(p); err == nil && info.Mode().IsDir() {
+	} else if info, err := os.Lstat(p); err == nil && info.Mode().IsDir() && info.Mode()&fs.ModeSymlink == 0 {
 		p += "/"
 	}
+	if isCur {
+		p += "./"
+	}
 	return p
 }
 
-func validFile(info os.FileInfo) bool {
-	if info.Mode().IsRegular() && len(info.Name()) > 0 && (hidden || info.Name()[0] != '.') {
-		if pattern != nil && !pattern.MatchString(info.Name()) {
-			return false
-		}
-
-		if !sync {
-			ext := path.Ext(info.Name())
-			if len(ext) > 0 {
-				ext = ext[1:]
-			}
-
-			if _, ok := filetypeMime[ext]; !ok {
-				return false
-			}
-		}
+func fileMatches(filename string) bool {
+	if pattern != nil && !pattern.MatchString(filename) {
+		return false
+	} else if mimetype != "" {
 		return true
 	}
-	return false
+
+	ext := path.Ext(filename)
+	if 0 < len(ext) {
+		ext = ext[1:]
+	}
+	if _, ok := filetypeMime[ext]; !ok {
+		return false
+	}
+	return true
 }
 
-func validDir(info os.FileInfo) bool {
-	return info.Mode().IsDir() && len(info.Name()) > 0 && (hidden || info.Name()[0] != '.')
+type DirEntry struct {
+	fs.FileInfo
 }
 
-func createTasks(inputs []string, output string) ([]Task, []string) {
+func (d DirEntry) Type() fs.FileMode {
+	return d.Mode().Type()
+}
+
+func (d DirEntry) Info() (fs.FileInfo, error) {
+	return d.FileInfo, nil
+}
+
+func createTasks(inputs []string, output string) ([]Task, []string, error) {
 	tasks := []Task{}
 	roots := []string{}
 	for _, input := range inputs {
-		input = sanitizePath(input)
-		info, err := os.Stat(input)
+		var info fs.FileInfo
+		var err error
+		if !preserveSymlinks {
+			info, err = os.Stat(input)
+		} else {
+			info, err = os.Lstat(input)
+		}
 		if err != nil {
-			Error.Fatalln(err)
+			return nil, nil, err
 		}
 
-		if info.Mode().IsRegular() {
-			tasks = append(tasks, NewTask("", input, output))
-		} else if info.Mode().IsDir() {
-			roots = append(roots, input)
-			if !recursive {
-				infos, err := ioutil.ReadDir(input)
+		if info.Mode()&fs.ModeSymlink != 0 {
+			if !sync {
+				Warning.Println("--sync not specified, omitting symbolic link", input)
+				continue
+			}
+			task, err := NewTask(path.Dir(input), input, output, true)
+			if err != nil {
+				return nil, nil, err
+			}
+			tasks = append(tasks, task)
+		} else if info.Mode().IsRegular() {
+			valid := pattern == nil || pattern.MatchString(info.Name())
+			if valid || sync {
+				task, err := NewTask(path.Dir(input), input, output, !valid)
 				if err != nil {
-					Error.Fatalln(err)
+					return nil, nil, err
 				}
-				for _, info := range infos {
-					if validFile(info) {
-						tasks = append(tasks, NewTask(input, path.Join(input, info.Name()), output))
+				tasks = append(tasks, task)
+			}
+		} else if info.Mode().IsDir() {
+			if !recursive {
+				Warning.Println("--recursive not specified, omitting directory", input)
+				continue
+			}
+			roots = append(roots, input)
+
+			var walkFn func(string, fs.DirEntry, error) error
+			walkFn = func(path string, d fs.DirEntry, err error) error {
+				if err != nil {
+					return err
+				} else if d.Name() == "." || d.Name() == ".." {
+					return nil
+				} else if len(d.Name()) == 0 || !hidden && d.Name()[0] == '.' {
+					if d.IsDir() {
+						return fs.SkipDir
 					}
+					return nil
 				}
-			} else {
-				err := filepath.Walk(input, func(path string, info os.FileInfo, err error) error {
+				path = sanitizePath(path)
+
+				if !preserveSymlinks && d.Type()&fs.ModeSymlink != 0 {
+					// follow and dereference symlinks
+					info, err := os.Stat(path)
 					if err != nil {
 						return err
 					}
-					if validFile(info) {
-						tasks = append(tasks, NewTask(input, path, output))
-					} else if info.Mode().IsDir() && !validDir(info) && info.Name() != "." && info.Name() != ".." { // check for IsDir, so we don't skip the rest of the directory when we have an invalid file
-						return filepath.SkipDir
+					if info.IsDir() {
+						return fs.WalkDir(os.DirFS(input), path, walkFn)
 					}
-					return nil
-				})
-				if err != nil {
-					Error.Fatalln(err)
+					d = DirEntry{info}
 				}
+
+				if preserveSymlinks && d.Type()&fs.ModeSymlink != 0 {
+					// copy symlinks as is
+					if !sync {
+						Warning.Println("--sync not specified, omitting symbolic link", path)
+						return nil
+					}
+					task, err := NewTask(input, path, output, true)
+					if err != nil {
+						return err
+					}
+					tasks = append(tasks, task)
+				} else if d.Type().IsRegular() {
+					valid := fileMatches(d.Name())
+					if valid || sync {
+						task, err := NewTask(input, path, output, !valid)
+						if err != nil {
+							return err
+						}
+						tasks = append(tasks, task)
+					}
+				}
+				return nil
+			}
+			if err := fs.WalkDir(os.DirFS("."), path.Clean(input), walkFn); err != nil {
+				return nil, nil, err
 			}
 		} else {
-			Error.Fatalln("not a file or directory", input)
+			return nil, nil, fmt.Errorf("not a file or directory %s", input)
 		}
 	}
-	return tasks, roots
+	return tasks, roots, nil
 }
 
 func openInputFile(input string) (io.ReadCloser, error) {
@@ -499,30 +649,65 @@ func openOutputFile(output string) (*os.File, error) {
 	return w, nil
 }
 
-func minify(mimetype string, t Task) bool {
-	if mimetype == "" {
-		for _, src := range t.srcs {
-			if len(path.Ext(src)) > 0 {
-				srcMimetype, ok := filetypeMime[path.Ext(src)[1:]]
-				if !ok && sync {
-					break // is sync==true, then len(t.srcs)==1
-				} else if !ok {
-					Error.Println("cannot infer mimetype from extension in", src)
-					return false
-				}
-				if mimetype == "" {
-					mimetype = srcMimetype
-				} else if srcMimetype != mimetype {
-					Error.Println("inferred mimetype", srcMimetype, "of", src, "for concatenation unequal to previous mimetypes", mimetype)
-					return false
-				}
+func createSymlink(input, output string) error {
+	if _, err := os.Stat(output); err == nil {
+		if err = os.Remove(output); err != nil {
+			return err
+		}
+	}
+	if err := os.MkdirAll(path.Dir(output), 0777); err != nil {
+		return err
+	}
+	if err := os.Symlink(input, output); err != nil {
+		return err
+	}
+	return nil
+}
+
+func minify(t Task) bool {
+	// synchronizing files that are not minified but just copied to the same directory, no action needed
+	if t.sync {
+		if t.srcs[0] == t.dst {
+			return true
+		} else if info, err := os.Lstat(t.srcs[0]); preserveSymlinks && err == nil && info.Mode()&fs.ModeSymlink != 0 {
+			src, err := os.Readlink(t.srcs[0])
+			if err != nil {
+				Error.Println(err)
+				return false
 			}
+			if err := createSymlink(src, t.dst); err != nil {
+				Error.Println(err)
+				return false
+			}
+			return true
 		}
 	}
 
-	// synchronizing files that are not minified but just copied to the same directory, no action needed
-	if sync && mimetype == "" && t.srcs[0] == t.dst {
-		return true
+	srcInfo, err := os.Stat(t.srcs[0])
+	if err != nil {
+		Error.Println(err)
+		return false
+	}
+
+	fileMimetype := mimetype
+	if mimetype == "" && !t.sync {
+		for _, src := range t.srcs {
+			ext := path.Ext(src)
+			if 0 < len(ext) {
+				ext = ext[1:]
+			}
+			srcMimetype, ok := filetypeMime[ext]
+			if !ok {
+				Error.Println("cannot infer mimetype from extension in", src, ", set --type or --mime explicitly")
+				return false
+			}
+			if fileMimetype == "" {
+				fileMimetype = srcMimetype
+			} else if srcMimetype != fileMimetype {
+				Error.Println("inferred mimetype", srcMimetype, "of", src, "for concatenation unequal to previous mimetypes, set --type or --mime explicitly")
+				return false
+			}
+		}
 	}
 
 	srcName := strings.Join(t.srcs, " + ")
@@ -553,31 +738,25 @@ func minify(mimetype string, t Task) bool {
 		}
 	}
 
-	fr, err := NewConcatFileReader(t.srcs, openInputFile)
+	fr, err := newConcatFileReader(t.srcs, openInputFile)
 	if err != nil {
 		Error.Println(err)
 		return false
 	}
-	if mimetype == filetypeMime["js"] {
+	if fileMimetype == filetypeMime["js"] {
 		fr.SetSeparator([]byte("\n"))
 	}
-	r := NewCountingReader(fr)
-
 	fw, err := openOutputFile(t.dst)
 	if err != nil {
 		Error.Println(err)
 		fr.Close()
 		return false
 	}
-	var w *countingWriter
-	if fw == os.Stdout {
-		w = NewCountingWriter(fw)
-	} else {
-		w = NewCountingWriter(bufio.NewWriter(fw))
-	}
+
+	os.Chmod(t.dst, srcInfo.Mode().Perm())
 
 	// synchronize file
-	if sync && mimetype == "" {
+	if t.sync {
 		_, err = io.Copy(fw, fr)
 		fr.Close()
 		fw.Close()
@@ -589,20 +768,31 @@ func minify(mimetype string, t Task) bool {
 		return true
 	}
 
+	r := newCountingReader(fr)
+	var w *countingWriter
+	if fw == os.Stdout {
+		w = newCountingWriter(fw)
+	} else {
+		w = newCountingWriter(bufio.NewWriter(fw))
+	}
+
 	success := true
 	startTime := time.Now()
-	if err = m.Minify(mimetype, w, r); err != nil {
+	if err = m.Minify(fileMimetype, w, r); err != nil {
+		if t.dst == "" {
+			fmt.Fprintf(os.Stdout, "\n\n")
+		}
 		Error.Println("cannot minify "+srcName+":", err)
 		success = false
 	}
 	if verbose {
 		dur := time.Since(startTime)
 		speed := "Inf MB"
-		if dur > 0 {
+		if 0 < dur {
 			speed = humanize.Bytes(uint64(float64(r.N) / dur.Seconds()))
 		}
 		ratio := 1.0
-		if r.N > 0 {
+		if 0 < r.N {
 			ratio = float64(w.N) / float64(r.N)
 		}
 
