@@ -35,17 +35,18 @@ func TestDataURI(t *testing.T) {
 		dataURI  string
 		expected string
 	}{
+		{"datx:x", "datx:x"},
 		{"data:,text", "data:,text"},
 		{"data:text/plain;charset=us-ascii,text", "data:,text"},
 		{"data:TEXT/PLAIN;CHARSET=US-ASCII,text", "data:,text"},
 		{"data:text/plain;charset=us-asciiz,text", "data:;charset=us-asciiz,text"},
 		{"data:;base64,dGV4dA==", "data:,text"},
-		{"data:text/svg+xml;base64,PT09PT09", "data:text/svg+xml;base64,PT09PT09"},
+		{"data:text/svg+xml;base64,IyMjIyMj", "data:text/svg+xml;base64,IyMjIyMj"},
 		{"data:text/xml;version=2.0,content", "data:text/xml;version=2.0,content"},
 		{"data:text/xml; version = 2.0,content", "data:text/xml;version=2.0,content"},
-		{"data:,%3D%3D%3D%3D%3D", "data:,%3D%3D%3D%3D%3D"},
-		{"data:,%3D%3D%3D%3D%3D%3D", "data:;base64,PT09PT09"},
-		{"data:text/x,<?xx?>", "data:text/x,%3C%3Fxx%3F%3E"},
+		{"data:,%23%23%23%23%23", "data:,%23%23%23%23%23"},
+		{"data:,%23%23%23%23%23%23", "data:;base64,IyMjIyMj"},
+		{"data:text/x,<?xx?>", "data:text/x,%3C?xx?%3E"},
 		{"data:text/other,\"<\u2318", "data:text/other,%22%3C%E2%8C%98"},
 		{"data:text/other,\"<\u2318>", "data:text/other;base64,IjzijJg+"},
 		{`data:text/svg+xml,%3Csvg height="100" width="100"><circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red" /></svg>`, `data:text/svg+xml,%3Csvg height="100" width="100"><circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red" /></svg>`},
@@ -70,6 +71,7 @@ func TestDecimal(t *testing.T) {
 		number   string
 		expected string
 	}{
+		{"", ""},
 		{"0", "0"},
 		{".0", "0"},
 		{"1.0", "1"},
@@ -149,6 +151,7 @@ func TestNumber(t *testing.T) {
 		number   string
 		expected string
 	}{
+		{"", ""},
 		{"0", "0"},
 		{".0", "0"},
 		{"1.0", "1"},
@@ -162,7 +165,12 @@ func TestNumber(t *testing.T) {
 		{"0.001", ".001"},
 		{"0.0001", "1e-4"},
 		{"100e1", "1e3"},
+		{"1e10", "1e10"},
+		{"1e-10", "1e-10"},
+		{"1000e-7", "1e-4"},
+		{"1000e-6", ".001"},
 		{"1.1e+1", "11"},
+		{"1.1e-1", ".11"},
 		{"1.1e6", "11e5"},
 		{"1.1e", "1.1e"},   // broken number, don't parse
 		{"1.1e+", "1.1e+"}, // broken number, don't parse
