@@ -27,6 +27,8 @@ func TestCSS(t *testing.T) {
 		{"@import 'file';", "@import 'file'"},
 		{"@import url('file');", "@import 'file'"},
 		{"@import url(//url);", `@import "//url"`},
+		{"@import url(\n//url\n);", `@import "//url"`},
+		{"@import url();", `@import ""`},
 		{"@font-face { x:y; }", "@font-face{x:y}"},
 
 		{"input[type=\"radio\"]{x:y}", "input[type=radio]{x:y}"},
@@ -108,7 +110,7 @@ func TestCSSInline(t *testing.T) {
 		{"color: rgba(255,0,0,1);", "color:red"},
 		{"color: rgba(255,0,0,2);", "color:red"},
 		{"color: rgba(255,0,0,0.5);", "color:rgba(255,0,0,.5)"},             // {"color: rgba(255,0,0,0.5);", "color:#ff000080"},
-		{"color: rgba(255,0,0,-1);", "color:transparent"},                   // {"color: rgba(255,0,0,-1);", "color:#0000"},
+		{"color: rgba(0,0,0,-1);", "color:transparent"},                     // {"color: rgba(255,0,0,-1);", "color:#0000"},
 		{"color: rgba(0%,0%,0%,0.2);", "color:rgba(0,0,0,.2)"},              // {"color: rgba(0%,15%,25%,0.2);", "color:#00264033"},
 		{"color: rgba(0,0,0,0.5);", "color:rgba(0,0,0,.5)"},                 // {"color: rgba(0,0,0,0.5);", "color:#00000080"},
 		{"color: rgba(0,0,0,0.264705882);", "color:rgba(0,0,0,.264705882)"}, // {"color: rgba(0,0,0,0.264705882);", "color:#0004"},
@@ -125,7 +127,7 @@ func TestCSSInline(t *testing.T) {
 		{"color: hsl(0,100%,50%);", "color:red"},
 		{"color: hsl(-360,100%,50%);", "color:red"},
 		{"color: hsla(1,2%,3%,1);", "color:#080807"},
-		{"color: hsla(1,2%,3%,0);", "color:transparent"}, // {"color: hsla(1,2%,3%,0);", "color:#0000"},
+		{"color: hsla(0,0%,0%,0);", "color:transparent"}, // {"color: hsla(1,2%,3%,0);", "color:#0000"},
 		{"color: hsl(48,100%,50%);", "color:#fc0"},
 		{"color: hsla(0 100% 50% / 1);", "color:red"},
 		{"color: hsla(0 100% 50% / 60%);", "color:hsla(0 100% 50%/.6)"},
@@ -157,15 +159,18 @@ func TestCSSInline(t *testing.T) {
 		{"background-position:center top 1%", "background-position:50% 1%"},
 		{"background-position:right 0 top 0", "background-position:100% 0"},
 		{"background-position:center 0px center 0%", "background-position:50%"},
+		{"background-position:center 0px center 0%,,right 0 top 0", "background-position:50%,,100% 0"},
 		{"background-repeat:space space", "background-repeat:space"},
 		{"background-repeat:round round", "background-repeat:round"},
 		{"background-repeat:repeat repeat", "background-repeat:repeat"},
 		{"background-repeat:no-repeat no-repeat", "background-repeat:no-repeat"},
 		{"background-repeat:repeat no-repeat", "background-repeat:repeat-x"},
 		{"background-repeat:no-repeat repeat", "background-repeat:repeat-y"},
+		{"background-repeat:no-repeat repeat,,repeat no-repeat", "background-repeat:repeat-y,,repeat-x"},
 		{"background-size:auto auto", "background-size:auto"},
 		{"background-size:30% auto", "background-size:30%"},
 		{"background-size:200px auto", "background-size:200px"},
+		{"background-size:200px auto,,30% auto", "background-size:200px,,30%"},
 		{"background:red", "background:red"},
 		{"background:red none", "background:red"},
 		{"background:red none 0 0", "background:red"},
@@ -176,6 +181,7 @@ func TestCSSInline(t *testing.T) {
 		{"background:#0000 none padding-box 0 0 / auto auto scroll border-box repeat repeat", "background:0 0"},
 		{"background:0 0 / auto", "background:0 0"},
 		{"background:0 0 / auto 10%", "background:0 0/auto 10%"},
+		{"background:0 / auto 10%", "background:0/auto 10%"},
 		{"background:0 0/200px auto", "background:0 0/200px"},
 		{"background:0 0 no-repeat", "background:no-repeat"},
 		{"background:0% 0%", "background:0 0"},
@@ -190,7 +196,13 @@ func TestCSSInline(t *testing.T) {
 		{"background:calc(5%-2%) center", "background:calc(5%-2%)"},
 		{"background:0 0 / 80% no-repeat url('firefox-logo.svg'), white 0 0 url('lizard.png');", "background:0 0/80% no-repeat url(firefox-logo.svg),#fff url(lizard.png)"},
 		{"background:rgba(255,0,0,1) url(foo.svg) no-repeat right .75rem center / auto calc(100% - 1.5rem)", "background:red url(foo.svg)no-repeat right .75rem center/auto calc(100% - 1.5rem)"},
-		{"box-shadow:rgba(11,22,33,0) 0 8px", "box-shadow:transparent 0 8px"},
+		{"background:left top,#fff / 5% auto", "background:0 0,#fff/5%"},
+		{"box-shadow:0 0 0 0", "box-shadow:0 0"},
+		{"box-shadow:0 0 0 0,0 0 0 0", "box-shadow:0 0,0 0"},
+		{"box-shadow:0 inset 0 0 blue 0", "box-shadow:0 inset 0 blue"},
+		{"box-shadow:rgba(0,0,0,0) 0 8px", "box-shadow:transparent 0 8px"},
+		{"box-shadow: inset .5em 0,, .39em 0", "box-shadow:inset .5em 0,,.39em 0"},
+		{"box-shadow:initial", "box-shadow:none"},
 		{"font-weight: normal;", "font-weight:400"},
 		{"font-weight: bold;", "font-weight:700"},
 		{"font: ;", "font:"},
@@ -205,6 +217,7 @@ func TestCSSInline(t *testing.T) {
 		{"font:medium/normal 'Arial'", "font:medium arial"},
 		{"font-family:'Arial', 'Times New Roman';", "font-family:arial,times new roman"},
 		{"font-family:'a  b';", "font-family:'a  b'"},
+		{"font-family:' a b ';", "font-family:' a b '"},
 		{"outline: none;", "outline:none"},
 		{"outline: solid black 0;", "outline:solid #000 0"},
 		{"outline: none black medium;", "outline:#000"},
@@ -217,11 +230,16 @@ func TestCSSInline(t *testing.T) {
 		{"border-color: currentcolor currentcolor currentcolor;", "border-color:initial"},
 		{"border-color: red red red;", "border-color:red"},
 		{"border-left-color: currentcolor;", "border-left-color:initial"},
+		{"border-left-color: black;", "border-left-color:#000"},
+		{"border: medium none;", "border:none"}, // #294
 		{"outline-color: white;", "outline-color:#fff"},
 		{"column-rule: medium currentcolor none;", "column-rule:none"},
+		{"column-rule: medium white none;", "column-rule:#fff"},
 		{"text-shadow: white 5px 5px;", "text-shadow:#fff 5px 5px"},
 		{"text-decoration: currentcolor none solid", "text-decoration:none"},
+		{"text-decoration: white none solid", "text-decoration:#fff"},
 		{"text-emphasis: none currentcolor", "text-emphasis:none"},
+		{"text-emphasis: none white", "text-emphasis:#fff"},
 		{"margin: 1 1 1 1;", "margin:1"},
 		{"margin: 1 2 1 2;", "margin:1 2"},
 		{"margin: 1 2 3 2;", "margin:1 2 3"},
@@ -238,14 +256,16 @@ func TestCSSInline(t *testing.T) {
 		{"flex:5 1 0", "flex:5"},
 		{"flex:5 1 0%", "flex:5"},
 		{"flex:5 1 0px", "flex:5"},
-		{"flex:5 0 0%", "flex:5 0 0"},
-		{"flex:5 0 0px", "flex:5 0 0"},
-		{"flex:0 1 auto", "flex:0 1"},
-		{"flex:none", "flex:0 0"},
-		{"flex:auto", "flex:1 1"},
-		{"flex:initial", "flex:0 1"},
-		{"flex:initial", "flex:0 1"},
-		{"flex:5 auto", "flex:5 1"},
+		{"flex:5 0 0%", "flex:5 0"},
+		{"flex:5 0 0px", "flex:5 0"},
+		{"flex:0 1 auto", "flex:initial"},
+		{"flex:1 1 auto", "flex:auto"},
+		{"flex:0 0 auto", "flex:none"},
+		{"flex:0 0 5000%", "flex:0 0 5e3%"},
+		{"flex:initial", "flex:initial"},
+		{"flex:auto", "flex:auto"},
+		{"flex:none", "flex:none"},
+		{"flex:5 auto", "flex:5 auto"},
 		{"flex:5 0px", "flex:5"},
 		{"flex:5 0%", "flex:5"},
 		{"flex:5 0", "flex:5 0"},
@@ -266,7 +286,7 @@ func TestCSSInline(t *testing.T) {
 		{"margin:05em", "margin:5em"},
 		{"margin:.50em", "margin:.5em"},
 		{"margin:5.0em", "margin:5em"},
-		{"margin:5000em", "margin:5e3em"},
+		{"margin:5000em", "margin:5e3em"}, // CR: CSS Values and Units Module Level 3
 		{"color:#c0c0c0", "color:silver"},
 		{"-ms-filter: \"progid:DXImageTransform.Microsoft.Alpha(Opacity=80)\";", "-ms-filter:\"alpha(opacity=80)\""},
 		{"filter: progid:DXImageTransform.Microsoft.Alpha(Opacity = 80);", "filter:alpha(opacity=80)"},
@@ -280,12 +300,25 @@ func TestCSSInline(t *testing.T) {
 		{"background:url('data:\\'\",text')", "background:url('data:\\'\",text')"},
 		{"margin:0 0 18px 0;", "margin:0 0 18px"},
 		{"z-index:1000", "z-index:1000"},
-		{"box-shadow:0 0 0 0", "box-shadow:0 0"},
-		{"box-shadow:0 inset 0 0 blue 0", "box-shadow:0 inset 0 blue"},
 		//{"flex:0px", "flex:0q"}, // TODO
 		{"g:url('abc\\\ndef')", "g:url(abcdef)"},
 		{"url:local('abc\\\ndef')", "url:local(abcdef)"},
 		{"url:local('abc def') , url('abc def') format('truetype')", "url:local('abc def'),url('abc def')format('truetype')"},
+		{"unicode-range:U", "unicode-range:U"},
+		{"unicode-range:U+", "unicode-range:U+"},
+		{"unicode-range:U+0-04FF", "unicode-range:U+0-4FF"},
+		{"unicode-range:U+0-7F", "unicode-range:U+0-7F"},
+		{"unicode-range:U+000-FFF", "unicode-range:U+???"},
+		{"unicode-range:U+1000-2FFF", "unicode-range:U+1000-2FFF"},
+		{"unicode-range:U+2300-23FF,U+2A00-2aFf", "unicode-range:U+23??,U+2A??"},
+		{"unicode-range:U+25-FF", "unicode-range:U+25-FF"},
+		{"unicode-range:U+26", "unicode-range:U+26"},
+		{"unicode-range:U+2600-26FF", "unicode-range:U+26??"},
+		{"unicode-range:u+1234-1234", "unicode-range:U+1234"},
+		{"unicode-range:U+26??,U+27??", "unicode-range:U+2600-27FF"},
+		{"unicode-range:U+26??,U+2680-2780", "unicode-range:U+2600-2780"},
+		{"unicode-range:U+26??,U+2680-2690", "unicode-range:U+26??"},
+		{"unicode-range:U+0-10FFFF", "unicode-range:initial"},
 
 		// case
 		{"MARGIN:1EM", "margin:1em"},
@@ -297,15 +330,18 @@ func TestCSSInline(t *testing.T) {
 		{"margin:calc(10px) calc(20px)", "margin:calc(10px)calc(20px)"},
 		{"border-left:0 none", "border-left:0"},
 		{"--custom-variable:0px;", "--custom-variable:0px"},
-		{"--foo: if(x > 5) this.width = 10", "--foo: if(x > 5) this.width = 10"},
-		{"--foo: ;", "--foo: "},
+		{"--foo: 0px ;", "--foo:0px"},
+		{"--foo: if(x > 5) this.width = 10", "--foo:if(x > 5) this.width = 10"},
+		{"--foo: ;", "--foo: "},               // whitespace value
+		{"--foo:;", "--foo:"},                 // empty value
+		{"--foo: initial ;", "--foo:initial"}, // invalid value, serializes to empty
 		{"color=blue;", "color=blue"},
 		{"x: white , white", "x:white,white"},
 
 		// TODO: functions
-		{"width:calc(0%-0px)", "width:calc(0%0)"}, // invalid
-		{"width:calc(0% - 0px)", "width:calc(0% - 0)"},
-		{"width:calc(calc(0% - 0px) + 1em)", "width:calc(calc(0% - 0) + 1em)"},
+		{"width:calc(0%-0px)", "width:calc(0%0px)"}, // invalid
+		{"width:calc(0% - 0px)", "width:calc(0% - 0px)"},
+		{"width:calc(calc(0% - 0px) + 1em)", "width:calc(calc(0% - 0px) + 1em)"},
 		//{"width:calc(5px);", "width:5px"},
 		//{"width:calc(5px - 3px);", "width:2px"},
 		//{"width:calc(5px + -3px);", "width:2px"},
@@ -354,17 +390,28 @@ func TestCSSInline(t *testing.T) {
 		{"color: rgb(0, 1, ident);", "color:rgb(0,1,ident)"},
 		{"color: rgb(ident);", "color:rgb(ident)"},
 		{"color: hsl(0,-1%,-1%);", "color:#000"},
+		{"color: hsl(-180,100%,50%);", "color:#0ff"},
 		{"margin: rgb(ident);", "margin:rgb(ident)"},
 		{"filter: progid:b().c.Alpha(rgba(x));", "filter:progid:b().c.Alpha(rgba(x))"},
 		{"margin: rgb((brackets));", "margin:rgb((brackets))"},
+		//{`background-color:transparent`, `background-color:#0000`}, // TODO for CSS3
 
 		// bugs
 		{"background: linear-gradient(-180deg, #355FFF 0%, #1F52FF 100%) 0% 0% / cover", "background:linear-gradient(-180deg,#355FFF 0%,#1F52FF 100%)0 0/cover"}, // #263
-		{"font:1em -apple-system", "font:1em '-apple-system'"}, // support for IE9, IE10, IE11, fixes #251
-		{"font:1em -", "font:1em '-'"},                         // support for IE9, IE10, IE11, fixes #251
+		{"font:1em -apple-system", "font:1em '-apple-system'"},     // support for IE9, IE10, IE11, fixes #251
+		{"font:1em -", "font:1em '-'"},                             // support for IE9, IE10, IE11, fixes #251
+		{"color:rgba(255,255,255,0)", "color:rgba(255,255,255,0)"}, // #327
+		{"box-shadow:none", "box-shadow:none"},                     // #332
+		{"margin:var(--a,0px)", "margin:var(--a,0px)"},             // #371
 
 		// go-fuzz
 		{"FONT-FAMILY: ru\"", "font-family:ru\""},
+		{`d:hsl(0033333333333333333333333333333333333333333333333333333333333333333333333333333200,040000199823736,2444)`, `d:hsl(33333333333333333333333333333333333333333333333333333333333333333333333333333200,40000199823736,2444)`},
+		{`d:hsl(0033333333333333333333333333333333333333333333333333333333333333333333333333333200%,040000199823736,2444)`, `d:hsl(33333333333333333333333333333333333333333333333333333333333333333333333333333200%,40000199823736,2444)`},
+		{`d:hsl(-360000000000000000000000000000,50%,50%)`, `d:#bf4040`},
+		{`d:hsl(360000000000000000000000000000,50%,50%)`, `d:#bf4040`},
+		{`background:none,,,,,,,,,,,,,,,#00ff00`, `background:0 0,,,,,,,,,,,,,,,#0f0`},
+		{`background:rgba(100%)`, `background:rgba(100%)`},
 	}
 
 	m := minify.New()
@@ -384,7 +431,10 @@ func TestCSSKeepCSS2(t *testing.T) {
 		css      string
 		expected string
 	}{
+		{`margin:5000`, `margin:5000`},
+		{`margin:5000%`, `margin:5000%`},
 		{`margin:5000em`, `margin:5000em`},
+		{`color:transparent`, `color:transparent`},
 	}
 
 	m := minify.New()
