@@ -27,6 +27,7 @@ func TestHTML(t *testing.T) {
 		{`html`, `html`},
 		{`<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML+RDFa 1.0//EN" "http://www.w3.org/MarkUp/DTD/xhtml-rdfa-1.dtd">`, `<!doctype html>`},
 		{`<!-- comment -->`, ``},
+		{`<!--# SSI Tag -->`, `<!--# SSI Tag -->`},
 		{`<style><!--\ncss\n--></style>`, `<style><!--\ncss\n--></style>`},
 		{`<style>&</style>`, `<style>&</style>`},
 		{`<html><head></head><body>x</body></html>`, `x`},
@@ -54,7 +55,7 @@ func TestHTML(t *testing.T) {
 		{`<span selected="selected"></span>`, `<span selected></span>`},
 		{`<noscript><html><img id="x"></noscript>`, `<noscript><img id=x></noscript>`},
 		{`<body id="main"></body>`, `<body id=main>`},
-		{`<link href="data:text/plain, data">`, `<link href=data:,+data>`},
+		{`<link href="data:text/plain, data">`, `<link href="data:, data">`},
 		{`<svg width="100" height="100"><circle cx="50" cy="50" r="40" stroke="green" stroke-width="4" fill="yellow" /></svg>`, `<svg width="100" height="100"><circle cx="50" cy="50" r="40" stroke="green" stroke-width="4" fill="yellow" /></svg>`},
 		{`<span ></span>`, `<span></span>`},
 		{`</span >`, `</span>`},
@@ -99,8 +100,14 @@ func TestHTML(t *testing.T) {
 		{"<x>\n<!--y-->\n</x>", "<x></x>"},
 		{"a <template> b </template> c", "a <template>b</template>c"},
 		{`<p class="  name  ">`, `<p class=name>`},
+		{`<p class="  name  other  ">`, `<p class="name other">`},
 		{`<p onclick="  javascript:lala  ">`, `<p onclick=lala>`},
-		{`<p url="  http://test  ">`, `<p url=http://test>`},
+		{`<p url="http://test">`, `<p url=http://test>`},
+		{`a<span> <img> </span>b`, `a<span> <img> </span>b`},
+		{`a <span> <img> </span>b`, `a <span><img> </span>b`},
+		{`a<picture> <img> </picture>b`, `a<picture> <img> </picture>b`},
+		{`a <picture> <img> </picture>b`, `a <picture><img> </picture>b`},
+		{`<input placeholder=" a " value=" b ">`, `<input placeholder=" a " value=" b ">`},
 
 		// from HTML Minifier
 		{`<DIV TITLE="blah">boo</DIV>`, `<div title=blah>boo</div>`},
@@ -116,6 +123,8 @@ func TestHTML(t *testing.T) {
 		{`<table><thead><tr><th>foo</th><th>bar</th></tr></thead><tfoot><tr><th>baz</th><th>qux</th></tr></tfoot><tbody><tr><td>boo</td><td>moo</td></tr></tbody></table>`,
 			`<table><thead><tr><th>foo<th>bar<tfoot><tr><th>baz<th>qux<tbody><tr><td>boo<td>moo</table>`},
 		{`<select><option>foo</option><option>bar</option></select>`, `<select><option>foo<option>bar</select>`},
+		{`<select><optgroup><option>foo</option></optgroup><optgroup><option>bar</option></optgroup><option>zoo</option></select>`, `<select><optgroup><option>foo<optgroup><option>bar</optgroup><option>zoo</select>`},
+		{`<select>text<option>foo</option>text<optgroup>text<option>bar</option>text</optgroup>text</select>`, `<select><option>foo<optgroup><option>bar</select>`},
 		{`<meta name="keywords" content="A, B">`, `<meta name=keywords content="A,B">`},
 		{`<iframe><html> <p> x </p> </html></iframe>`, `<iframe><p>x</iframe>`},
 		{`<math> &int;_a_^b^{f(x)<over>1+x} dx </math>`, `<math> &int;_a_^b^{f(x)<over>1+x} dx </math>`},
@@ -139,6 +148,10 @@ func TestHTML(t *testing.T) {
 		{`<script><!--<`, `<script><!--<`},
 
 		// bugs
+		{`<i class="fas"></i> Text`, `<i class=fas></i> Text`},                                                   // #390
+		{`<i class="fas"> </i> Text`, `<i class=fas></i>Text`},                                                   // #390
+		{`<span translate="no">`, `<span translate=no>`},                                                         // #352
+		{`<input title=""><input lang="">`, `<input title><input lang>`},                                         // #331
 		{`<amp-analytics type=adobeanalytics_nativeConfig>`, `<amp-analytics type=adobeanalytics_nativeConfig>`}, // #270
 		{`<p>text</p><br>text`, `<p>text</p><br>text`},                                                           // #122
 		{`text <img> text`, `text <img> text`},                                                                   // #89
@@ -173,8 +186,10 @@ func TestHTMLCSSJS(t *testing.T) {
 		expected string
 	}{
 		// bugs
+		{`<script>var a='<\/script>'</script>`, `<script>var a='<\/script>'</script>`},                                                          // #355
+		{`<script>var a='</scr'+'ipt>'</script>`, `<script>var a='<\/script>'</script>`},                                                        // #355
 		{`<div style="font-family: Arial, &#39;sans-serif&#39;; font-size: 22px;">`, `<div style=font-family:Arial,sans-serif;font-size:22px>`}, // #272
-		{`<style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;}</style>`, `<style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;}</style>`},
+		{`<style amp-boilerplate>body{-webkit-animation:-amp-start 8s    steps(1,end) 0s 1 normal both;}</style>`, `<style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;}</style>`},
 	}
 
 	m := minify.New()
