@@ -1,4 +1,4 @@
-package json // import "github.com/tdewolff/minify/json"
+package json
 
 import (
 	"bytes"
@@ -16,9 +16,15 @@ func TestJSON(t *testing.T) {
 		json     string
 		expected string
 	}{
+		{"", ""},
 		{"{ \"a\": [1, 2] }", "{\"a\":[1,2]}"},
 		{"[{ \"a\": [{\"x\": null}, true] }]", "[{\"a\":[{\"x\":null},true]}]"},
 		{"{ \"a\": 1, \"b\": 2 }", "{\"a\":1,\"b\":2}"},
+		{"1.3e1", "13"},
+		{"1E+03", "1e3"},
+		{"0.1", "0.1"},
+		{"-0.1", "-0.1"},
+		{"1.0", "1"},
 	}
 
 	m := minify.New()
