@@ -7,6 +7,7 @@ import (
 
 	"github.com/tdewolff/minify/v2"
 	"github.com/tdewolff/minify/v2/css"
+	minifyXML "github.com/tdewolff/minify/v2/xml"
 	"github.com/tdewolff/parse/v2"
 	"github.com/tdewolff/parse/v2/buffer"
 	"github.com/tdewolff/parse/v2/xml"
@@ -20,18 +21,16 @@ var (
 	pathBytes     = []byte("<path")
 	dBytes        = []byte("d")
 	zeroBytes     = []byte("0")
+	n100pBytes    = []byte("100%")
 	cssMimeBytes  = []byte("text/css")
+	noneBytes     = []byte("none")
 	urlBytes      = []byte("url(")
 )
 
 ////////////////////////////////////////////////////////////////
 
-// DEPRECATED: DefaultMinifier is the default minifier.
-var DefaultMinifier = &Minifier{}
-
 // Minifier is an SVG minifier.
 type Minifier struct {
-	Decimals     int // DEPRECATED
 	Precision    int // number of significant digits
 	newPrecision int // precision for new numbers
 }
@@ -43,12 +42,6 @@ func Minify(m *minify.M, w io.Writer, r io.Reader, params map[string]string) err
 
 // Minify minifies SVG data, it reads from r and writes to w.
 func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]string) error {
-	if o.Decimals != 0 {
-		minify.Warning.Println("SVG option `Decimals` is deprecated, using as `Precision` instead. Be aware that `Decimals` meant the number of digits behind the dot while `Precision` means the number of significant digits. Example: 1.23 with `Decimals=1` would give 1.2 but with `Pecision=1` gives 1. The default `Decimals=-1` is now `Precision=0` which prints the whole number.")
-	}
-	if o.Precision == 0 {
-		o.Precision = o.Decimals
-	}
 	o.newPrecision = o.Precision
 	if o.newPrecision <= 0 || 15 < o.newPrecision {
 		o.newPrecision = 15 // minimum number of digits a double can represent exactly
@@ -63,38 +56,39 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 	minifyBuffer := buffer.NewWriter(make([]byte, 0, 64))
 	attrByteBuffer := make([]byte, 0, 64)
 
-	l := xml.NewLexer(r)
-	defer l.Restore()
+	z := parse.NewInput(r)
+	defer z.Restore()
 
+	l := xml.NewLexer(z)
 	tb := NewTokenBuffer(l)
 	for {
 		t := *tb.Shift()
 		switch t.TokenType {
 		case xml.ErrorToken:
+			if _, err := w.Write(nil); err != nil {
+				return err
+			}
 			if l.Err() == io.EOF {
 				return nil
 			}
 			return l.Err()
 		case xml.DOCTYPEToken:
 			if len(t.Text) > 0 && t.Text[len(t.Text)-1] == ']' {
-				if _, err := w.Write(t.Data); err != nil {
-					return err
-				}
+				w.Write(t.Data)
 			}
 		case xml.TextToken:
-			t.Data = parse.ReplaceMultipleWhitespaceAndEntities(t.Data, xml.EntitiesMap, nil)
+			t.Data = parse.ReplaceMultipleWhitespaceAndEntities(t.Data, minifyXML.EntitiesMap, nil)
 			t.Data = parse.TrimWhitespace(t.Data)
 
 			if tag == Style && len(t.Data) > 0 {
 				if err := m.MinifyMimetype(defaultStyleType, w, buffer.NewReader(t.Data), defaultStyleParams); err != nil {
 					if err != minify.ErrNotExist {
 						return err
-					} else if _, err := w.Write(t.Data); err != nil {
-						return err
 					}
+					w.Write(t.Data)
 				}
-			} else if _, err := w.Write(t.Data); err != nil {
-				return err
+			} else {
+				w.Write(t.Data)
 			}
 		case xml.CDATAToken:
 			if tag == Style {
@@ -111,12 +105,9 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 			if t.Text, useText = xml.EscapeCDATAVal(&attrByteBuffer, t.Text); useText {
 				t.Text = parse.ReplaceMultipleWhitespace(t.Text)
 				t.Text = parse.TrimWhitespace(t.Text)
-
-				if _, err := w.Write(t.Text); err != nil {
-					return err
-				}
-			} else if _, err := w.Write(t.Data); err != nil {
-				return err
+				w.Write(t.Text)
+			} else {
+				w.Write(t.Data)
 			}
 		case xml.StartTagPIToken:
 			for {
@@ -134,8 +125,8 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 
 			if t.Data == nil {
 				skipTag(tb)
-			} else if _, err := w.Write(t.Data); err != nil {
-				return err
+			} else {
+				w.Write(t.Data)
 			}
 		case xml.AttributeToken:
 			if len(t.AttrVal) == 0 || t.Text == nil { // data is nil when attribute has been removed
@@ -149,27 +140,19 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 			}
 			if attr == Xml_Space && bytes.Equal(val, []byte("preserve")) ||
 				tag == Svg && (attr == Version && bytes.Equal(val, []byte("1.1")) ||
-					attr == X && bytes.Equal(val, []byte("0")) ||
-					attr == Y && bytes.Equal(val, []byte("0")) ||
-					attr == Width && bytes.Equal(val, []byte("100%")) ||
-					attr == Height && bytes.Equal(val, []byte("100%")) ||
+					attr == X && bytes.Equal(val, zeroBytes) ||
+					attr == Y && bytes.Equal(val, zeroBytes) ||
 					attr == PreserveAspectRatio && bytes.Equal(val, []byte("xMidYMid meet")) ||
-					attr == BaseProfile && bytes.Equal(val, []byte("none")) ||
+					attr == BaseProfile && bytes.Equal(val, noneBytes) ||
 					attr == ContentScriptType && bytes.Equal(val, []byte("application/ecmascript")) ||
-					attr == ContentStyleType && bytes.Equal(val, []byte("text/css"))) ||
-				tag == Style && attr == Type && bytes.Equal(val, []byte("text/css")) {
+					attr == ContentStyleType && bytes.Equal(val, cssMimeBytes)) ||
+				tag == Style && attr == Type && bytes.Equal(val, cssMimeBytes) {
 				continue
 			}
 
-			if _, err := w.Write(spaceBytes); err != nil {
-				return err
-			}
-			if _, err := w.Write(t.Text); err != nil {
-				return err
-			}
-			if _, err := w.Write(isBytes); err != nil {
-				return err
-			}
+			w.Write(spaceBytes)
+			w.Write(t.Text)
+			w.Write(isBytes)
 
 			if tag == Svg && attr == ContentStyleType {
 				val = minify.Mediatype(val)
@@ -223,9 +206,7 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 
 			// prefer single or double quotes depending on what occurs more often in value
 			val = xml.EscapeAttrVal(&attrByteBuffer, val)
-			if _, err := w.Write(val); err != nil {
-				return err
-			}
+			w.Write(val)
 		case xml.StartTagCloseToken:
 			next := tb.Peek(0)
 			skipExtra := false
@@ -239,28 +220,24 @@ func (o *Minifier) Minify(m *minify.M, w io.Writer, r io.Reader, _ map[string]st
 				if skipExtra {
 					tb.Shift()
 				}
-				if _, err := w.Write(voidBytes); err != nil {
-					return err
-				}
+				w.Write(voidBytes)
 			} else {
-				if _, err := w.Write(t.Data); err != nil {
-					return err
-				}
+				w.Write(t.Data)
+			}
+
+			if tag == ForeignObject {
+				printTag(w, tb, tag)
 			}
 		case xml.StartTagCloseVoidToken:
 			tag = 0
-			if _, err := w.Write(t.Data); err != nil {
-				return err
-			}
+			w.Write(t.Data)
 		case xml.EndTagToken:
 			tag = 0
 			if len(t.Data) > 3+len(t.Text) {
 				t.Data[2+len(t.Text)] = '>'
 				t.Data = t.Data[:3+len(t.Text)]
 			}
-			if _, err := w.Write(t.Data); err != nil {
-				return err
-			}
+			w.Write(t.Data)
 		}
 	}
 }
@@ -300,6 +277,39 @@ func (o *Minifier) shortenRect(tb *TokenBuffer, t *Token) {
 
 ////////////////////////////////////////////////////////////////
 
+func printTag(w io.Writer, tb *TokenBuffer, tag Hash) {
+	level := 0
+	inStartTag := false
+	for {
+		t := *tb.Peek(0)
+		switch t.TokenType {
+		case xml.ErrorToken:
+			return
+		case xml.StartTagToken:
+			inStartTag = t.Hash == tag
+			if t.Hash == tag {
+				level++
+			}
+		case xml.StartTagCloseVoidToken:
+			if inStartTag {
+				if level == 0 {
+					return
+				}
+				level--
+			}
+		case xml.EndTagToken:
+			if t.Hash == tag {
+				if level == 0 {
+					return
+				}
+				level--
+			}
+		}
+		w.Write(t.Data)
+		tb.Shift()
+	}
+}
+
 func skipTag(tb *TokenBuffer) {
 	level := 0
 	for {
